<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();
require_once($CFG->dirroot . '/course/format/topics/lib.php');

/**
 * format_iena
 *
 * @package    format_iena
 * @copyright  2018 Softia/Université lorraine
 * @author     vrignaud camille/Thomas Fradet
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class format_iena extends format_topics {
    /**
     * course_format_options
     *
     * @param bool $foreditform
     * @return array
     */
    public function course_format_options($foreditform = false) {
        global $PAGE;
        static $courseformatoptions = false;
        if ($courseformatoptions === false) {
            $courseformatoptions['allmodulesbreadcrum'] = array(
                'default' => 0,
                'type' => PARAM_INT,
            );
            $courseformatoptions['viewiconmessage'] = array(
                'default' => 1,
                'type' => PARAM_INT,
            );
            $courseformatoptions['enableclickicons'] = array(
                'default'=> 1,
                'type' => PARAM_INT,
            );
        }
        if ($foreditform && !isset($courseformatoptions['coursedisplay']['label'])) {
            $choicetab = array();
            $choicetab['1'] = get_string('yes', 'format_iena');
            $choicetab['0'] = get_string('no', 'format_iena');
            $courseformatoptionsedit['allmodulesbreadcrum'] = array(
                'label' => "Afficher les activités cachées mais disponibles dans la barre de progression.",
                'element_type' => 'select',
                'element_attributes' => array($choicetab),
            );
            $courseformatoptionsedit['viewiconmessage'] = array(
                'label' => get_string('hide_icon_message', 'format_iena'),
                'element_type' => 'select',
                'element_attributes' => array($choicetab),
            );
            $courseformatoptions['enableclickicons'] = array(
                'label' => get_string('enable_icon_click', 'format_iena'),
                'element_type' => 'select',
                'element_attributes' => array($choicetab),
                'help'=> 'enable_icon_click',
                'help_component'=>'format_iena',
            );
            $courseformatoptions = array_merge_recursive($courseformatoptions, $courseformatoptionsedit);
        }
        return $courseformatoptions;
    }

    /**
     * Definitions of the additional options that this course format uses for section
     *
     * See {@link format_base::course_format_options()} for return array definition.
     *
     * Additionally section format options may have property 'cache' set to true
     * if this option needs to be cached in {@link get_fast_modinfo()}. The 'cache' property
     * is recommended to be set only for fields used in {@link format_base::get_section_name()},
     * {@link format_base::extend_course_navigation()} and {@link format_base::get_view_url()}
     *
     * For better performance cached options are recommended to have 'cachedefault' property
     * Unlike 'default', 'cachedefault' should be static and not access get_config().
     *
     * Regardless of value of 'cache' all options are accessed in the code as
     * $sectioninfo->OPTIONNAME
     * where $sectioninfo is instance of section_info, returned by
     * get_fast_modinfo($course)->get_section_info($sectionnum)
     * or get_fast_modinfo($course)->get_section_info_all()
     *
     * All format options for particular section are returned by calling:
     * $this->get_format_options($section);
     *
     * @param bool $foreditform
     * @return array
     */
    public function section_format_options($foreditform = false) {
        global $CFG;
        static $courseformatoptions = false;

        if ($courseformatoptions === false) {
            $courseformatoptions = array(
                'presence' => array(
                    'label' => "Présence",
                    'type' => PARAM_INT
                ),
                'daterendu' => array(
                    'label' => "Date rendu",
                    'type' => PARAM_INT
                ),
                'daysnotif' => array(
                    'label' => "Notification",
                    'type' => PARAM_INT
                ),
                'msg' => array(
                    'label' => "Message",
                    'type' => PARAM_RAW
                ),
            );
        }

        return $courseformatoptions;
    }

    /**
     * Adds format options elements to the course/section edit form.
     *
     * This function is called from {@link course_edit_form::definition_after_data()}.
     *
     * @param MoodleQuickForm $mform form the elements are added to.
     * @param bool $forsection 'true' if this is a section edit form, 'false' if this is course edit form.
     * @return array array of references to the added form elements.
     */
    public function create_edit_form_elements(&$mform, $forsection = false) {
        global $COURSE;
        global $PAGE;

        $elements = parent::create_edit_form_elements($mform, $forsection);
        if ($forsection) {
            $mform->removeElement('presence', false);
            $mform->removeElement('daterendu', false);
            $mform->removeElement('daysnotif', false);
            $mform->removeElement('msg', false);

            $sectionclass = new stdClass();
            $sectionclass->id = optional_param('id', 0, PARAM_INT);
            $sectionconfig = $this->get_format_options($sectionclass);

            $mform->addElement('header', 'nameforyourheaderelement', 'Format hybride');

            // Modalité.
            $radioarray = array();
            $radioarray[] = $mform->createElement('radio', 'presence', '', 'Distance', 2);
            $radioarray[] = $mform->createElement('radio', 'presence', '', 'Présentiel', 1);
            $radioarray[] = $mform->createElement('radio', 'presence', '', 'Aucune', 0);
            $mform->addGroup($radioarray, 'modalite', 'Modalité', array(' '), false);
            $mform->addHelpButton('modalite', 'modalite', 'format_iena');

            // Date cours à distance.
            $datearray = array();
            $datearray[] = $mform->createElement('date_time_selector', 'daterendu', '');
            $datearray[] = $mform->createElement('checkbox', 'daterenducheck', 'Activer');
            $mform->addGroup($datearray, 'groupdate', 'Date d\'échéance', array(' '), false);
            $mform->disabledIf('groupdate', 'daterenducheck');
            // On montre le groupe seulement si à distance.
            $mform->hideIf('groupdate', 'presence', 'neq', 2);

            // Date cours présentiel.
            $datearray = array();
            $datearray[] = $mform->createElement('date_time_selector', 'daterendu2', '');
            $datearray[] = $mform->createElement('checkbox', 'daterenducheck2', 'Activer');
            $mform->addGroup($datearray, 'groupdate2', 'Date du cours', array(' '), false);
            $mform->disabledIf('groupdate2', 'daterenducheck2');
            // On montre le groupe si à présentiel.
            $mform->hideIf('groupdate2', 'presence', 'neq', 1);

            // Notification distanciel.
            $daysarray = array();
            $days = array(0 => 0, 1 => 1, 2 => 2, 3 => 3, 4 => 4, 5 => 5);
            $daysarray[] = $mform->createElement('select', 'daysnotif', "jours avant la séance", $days);
            $daysarray[] = $mform->createElement('html', ' <span id="days-text">jours avant la séance</span>');
            $daysarray[] = $mform->createElement('checkbox', 'selectdayscheck', 'Activer');
            $mform->addGroup($daysarray, 'groupdays', 'Notification', array(' '), false);
            $mform->disabledIf('groupdays', 'selectdayscheck');
            $mform->disabledIf('groupdays', 'daterenducheck');
            $mform->hideIf('groupdays', 'presence', 'neq', 2);
            $mform->hideIf('groupdays', 'daterenducheck');

            // Notification présentiel.
            $daysarray = array();
            $daysarray[] = $mform->createElement('select', 'daysnotif2', "jours avant le rendu", $days);
            $daysarray[] = $mform->createElement('html', ' <span id="days-text">jours avant le rendu</span>');
            $daysarray[] = $mform->createElement('checkbox', 'selectdayscheck2', 'Activer');
            $mform->addGroup($daysarray, 'groupdays2', 'Notification', array(' '), false);
            $mform->disabledIf('groupdays2', 'selectdayscheck2');
            $mform->disabledIf('groupdays2', 'daterenducheck2');
            $mform->hideIf('groupdays2', 'presence', 'neq', 1);
            $mform->hideIf('groupdays2', 'daterenducheck2');

            // Message présentiel.
            $msgpresgrp = array();
            $msgpresgrp[] = $mform->createElement('editor', 'msg_pres', get_string('message_pres', 'format_iena'));
            $mform->setType('msg_pres', PARAM_RAW);
            $mform->setDefault('msg_pres', array('text' => get_string('message_default_pres', 'format_iena')));
            $mform->addGroup($msgpresgrp, 'msg_pres_grp', get_string('message_pres', 'format_iena'), ' ', false);
            $mform->hideIf('msg_pres_grp', 'presence', 'neq', 1);
            $mform->hideIf('msg_pres_grp', 'selectdayscheck2');
            $mform->hideIf('msg_pres_grp', 'daterenducheck2');

            // Message distanciel.
            $msgdistgrp = array();
            $msgdistgrp[] = $mform->createElement('editor', 'msg_dist', get_string('message_dist', 'format_iena'));
            $mform->setType('msg_dist', PARAM_RAW);
            $mform->setDefault('msg_dist', array('text' => get_string('message_default_dist', 'format_iena')));
            $mform->addGroup($msgdistgrp, 'msg_dist_grp', get_string('message_dist', 'format_iena'), ' ', false);
            $mform->hideIf('msg_dist_grp', 'presence', 'neq', 2);
            $mform->hideIf('msg_dist_grp', 'selectdayscheck');
            $mform->hideIf('msg_dist_grp', 'daterenducheck');

            // On ne peut pas activer des notifs s'il n'y a pas de date de rendu/date de cours.
            // On prérempli le formulaire si des données existent en BDD.
            if ($sectionconfig['presence']) {
                $mform->setDefault('presence', $sectionconfig['presence']);
            }
            if ($sectionconfig['daterendu']) {
                // Cas distance.
                if (isset($sectionconfig['presence']) && $sectionconfig['presence'] == 2) {
                    $mform->setDefault('daterendu', $sectionconfig['daterendu']);
                    $mform->setDefault('daterenducheck', "1");
                    if (isset($sectionconfig['daysnotif']) && $sectionconfig['daysnotif'] > -1) {
                        $mform->setDefault('daysnotif', $sectionconfig['daysnotif']);
                        $mform->setDefault('selectdayscheck', "1");
                        $mform->setDefault('msg_dist', array('text' => $sectionconfig['msg']));
                    }
                } else if (isset($sectionconfig['presence']) && $sectionconfig['presence'] == 1) {
                    // Cas présence.
                    $mform->setDefault('daterendu2', $sectionconfig['daterendu']);
                    $mform->setDefault('daterenducheck2', "1");
                    if (isset($sectionconfig['daysnotif']) && $sectionconfig['daysnotif'] > -1) {
                        $mform->setDefault('daysnotif2', $sectionconfig['daysnotif']);
                        $mform->setDefault('selectdayscheck2', "1");
                        $mform->setDefault('msg_pres', array('text' => $sectionconfig['msg']));
                    }
                }
            }
        }
        return $elements;
    }

    /**
     * Updates format options for a course
     *
     * In case if course format was changed to 'periods', we try to copy options
     * 'coursedisplay', 'numsections' and 'hiddensections' from the previous format.
     * If previous course format did not have 'numsections' option, we populate it with the
     * current number of sections
     *
     * @param stdClass|array $data return value from {@link moodleform::get_data()} or array with data
     * @param stdClass $oldcourse if this function is called from {@link update_course()}
     *     this object contains information about the course before update
     * @return bool whether there were any changes to the options values
     */
    public function update_section_format_options($data) {
        global $DB;
        
        $objectsection = new stdClass();
        $objectsection->id = $data["id"];
        $sectionconfig = $this->get_format_options($objectsection);

        // If a date was set but we want to remove it.
        if ((!isset($data['daterenducheck']) && !isset($data['daterenducheck2'])) && (isset($data['daterendu']))) {
            if (array_key_exists('daterendu', $sectionconfig) && !empty($sectionconfig['daterendu'])) {
                $data['daterendu'] = null;
            } else {
                unset($data['daterendu']);
            }
        } else if (isset($data['daterenducheck2'])) {
            $data['daterendu'] = $data['daterendu2'];
        }

        if (isset($data['daysnotif']) && ($data['presence'] == 2)) {
            $data['msg'] = $data['msg_dist']['text'];

        } else if (isset($data['daysnotif2']) && ($data['presence'] == 1)) {
            $data['msg'] = $data['msg_pres']['text'];
        }

        if ($data['msg'] == "<p dir=\"ltr\" style=\"text-align: left;\"><br></p>") {
            $data['msg'] = "";
        }

        // De base daysnotifs est vide et daysnotifs2 n'existe pas.
        // Le système de notification n'a jamais été activé et on veut l'activer
        // Si c'est en présence.
        if (isset($data['daysnotif2']) && isset($data['selectdayscheck2']) && $data['presence'] == 1) {
            $data['daysnotif'] = $data["daysnotif2"];
        }

        // Le système de notif a déjà été activé  et on veut le désactiver OU il a jamais été activé et on ne veut pas l'activer.
        if (empty($data['daysnotif']) || (!isset($data['selectdayscheck']) && !isset($data['selectdayscheck2']))) {
            if (array_key_exists('daysnotif', $sectionconfig) && $sectionconfig['daysnotif'] > -1) {
                $data['daysnotif'] = -1;
                $data['msg'] = null;
            } else {
                unset($data['daysnotif']);
            }
        }

        return $this->update_format_options($data, $data['id']);
    }

    /**
     * get_view_url
     *
     * @param int|stdclass $section
     * @param array $options
     * @return null|moodle_url
     */
    public function get_view_url($section, $options = array()) {
        global $CFG;
        $course = $this->get_course();
        $url = new moodle_url('/course/view.php', array('id' => $course->id));

        $sr = null;
        if (array_key_exists('sr', $options)) {
            $sr = $options['sr'];
        }
        if (is_object($section)) {
            $sectionno = $section->section;
        } else {
            $sectionno = $section;
        }
        if ($sectionno !== null) {
            if ($sr !== null) {
                if ($sr) {
                    $usercoursedisplay = COURSE_DISPLAY_MULTIPAGE;
                    $sectionno = $sr;
                } else {
                    $usercoursedisplay = COURSE_DISPLAY_SINGLEPAGE;
                }
            } else {
                $usercoursedisplay = 0;
            }
            if ($sectionno != 0 && $usercoursedisplay == COURSE_DISPLAY_MULTIPAGE) {
                $url->param('section', $sectionno);
            } else {
                if (empty($CFG->linkcoursesections) && !empty($options['navigation'])) {
                    return null;
                }
                $url->set_anchor('section-' . $sectionno);
            }
        }
        return $url;
    }
}

/**
 * Implements callback inplace_editable() allowing to edit values in-place
 *
 * @param string $itemtype
 * @param int $itemid
 * @param mixed $newvalue
 * @return \core\output\inplace_editable
 */
function format_iena_inplace_editable($itemtype, $itemid, $newvalue) {
    global $DB, $CFG;
    require_once($CFG->dirroot . '/course/lib.php');
    if ($itemtype === 'sectionname' || $itemtype === 'sectionnamenl') {
        $section = $DB->get_record_sql('SELECT s.*
            FROM {course_sections} s
            JOIN {course} c ON s.course = c.id
            WHERE s.id = ?
            AND c.format = ?',
            array($itemid, 'iena'),
            MUST_EXIST
        );
        return course_get_format($section->course)->inplace_editable_update_section_name($section, $itemtype, $newvalue);
    }
}

/**
 * Extends the course administration navigation with the Badges page
 *
 * @param navigation_node $coursenode
 * @param object $course
 */
function iena_add_course_navigation(navigation_node $coursenode, stdClass $course) {
    echo "<br><br>br>";
    echo "ien add course";
}

function format_iena_add_course_navigation(navigation_node $coursenode, stdClass $course) {

    echo "<br><br>br>";
    echo "formatien add course";
}
